/*
 * Copyright (c) 2013 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __DBG_H__
#define __DBG_H__

#include <app.h>
#include <system_settings.h>
#include <Elementary.h>
#include <efl_extension.h>
#include <dlog.h>
#include <glib.h>
#include <system_info.h>
#include <geofence_manager.h>

#if !defined(PACKAGE)
#define PACKAGE "$(packageName)"
#endif

#ifdef  LOG_TAG
#undef  LOG_TAG
#endif
#define LOG_TAG "lbs-geofence"

#ifndef _ERR
#define _ERR(fmt, args...) LOGE("[%s:%d] "fmt"\n", __func__, __LINE__, ##args)
#endif

#ifndef _DBG
#define _DBG(fmt, args...) LOGD("[%s:%d] "fmt"\n", __func__, __LINE__, ##args)
#endif

#ifndef _INFO
#define _INFO(fmt, args...) LOGI("[%s:%d] "fmt"\n", __func__, __LINE__, ##args)
#endif

#define METHOD_GPS   0
#define METHOD_WIFI  1
#define METHOD_BT    2

#define MAX_COUNT 100

#define LATITUDE  12.98025
#define LONGITUDE 77.69773
#define BSSID     "88:30:8a:6c:28:15"

typedef struct appdata {
	Evas_Object *win;
	Evas_Object *conform;
	Evas_Object *layout;
	Evas_Object *nf;
	Evas_Object *entry;
	Evas_Object *event_entry;
	Evas_Object *status_entry;
	Evas_Object *basic_content;
	Evas_Object *popup;

	Evas_Object *create_btn;
	Evas_Object *fence_status_btn;
	Evas_Object *add_fence_btn;
	Evas_Object *add_place_btn;
	Evas_Object *start_btn;
	Evas_Object *stop_btn;
	Evas_Object *remove_fence_btn;
	Evas_Object *remove_place_btn;
	Evas_Object *update_place_btn;
	Evas_Object *destroy_btn;

	Evas_Object *gen_entry;
	Evas_Object *gen_entry_nd;

	int place_id[MAX_COUNT];
	int fence_id[MAX_COUNT];

	geofence_manager_h geo_manager;
} appdata_s;

#endif /* __DBG_H__ */
