/*
 * Copyright (c) 2013 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "lbs-geofence.h"

int myfence_index = 0;

static void
win_delete_request_cb(void *data , Evas_Object *obj , void *event_info)
{
	ui_app_exit();
}

static void
popup_timeout_cb(void *data, Evas_Object *obj, void *event_info)
{
	evas_object_del(obj);
}

static void
show_toast_popup(appdata_s *ad, char *str)
{
	Evas_Object *popup;

	popup = elm_popup_add(ad->win);
	elm_object_style_set(popup, "toast");
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_text_set(popup, str);
	elm_popup_timeout_set(popup, 2.0);
	evas_object_smart_callback_add(popup, "timeout", popup_timeout_cb, NULL);

	evas_object_show(popup);
}

static char*
gl_id_text_get_cb(void *data, Evas_Object *obj, const char *part)
{
	char buf[10] = {};
	snprintf(buf, sizeof(buf), "ID : %d", (int)data);
	return strdup(buf);
}

static char*
gl_method_text_get_cb(void *data, Evas_Object *obj, const char *part)
{
	switch ((int)data) {
	case METHOD_GPS:
		return strdup("GPS");
	case METHOD_WIFI:
		return strdup("Wi-Fi");
	case METHOD_BT:
		return strdup("Bluetooth");
	default:
		break;
	}
	return NULL;
}

static char*
gl_text_get_cb(void *data, Evas_Object *obj, const char *part)
{
	return strdup(data);
}

static Evas_Object *
gl_entry_content_get(void *data, Evas_Object *obj, const char *part)
{
	appdata_s *ad = evas_object_data_get(obj, "app_data");
	char *value = evas_object_data_get(obj, "value");
	char buf[20] = {};

	if (!strcmp(part, "elm.icon.entry")) {
		if (!strcmp(data, "Longitude : ")) {
			ad->gen_entry_nd = elm_entry_add(obj);
			elm_entry_single_line_set(ad->gen_entry_nd, EINA_TRUE);
			elm_entry_scrollable_set(ad->gen_entry_nd, EINA_TRUE);
			snprintf(buf, sizeof(buf), "%f", LONGITUDE);
			elm_entry_entry_set(ad->gen_entry_nd, buf);

			return ad->gen_entry_nd;
		} else {
			ad->gen_entry = elm_entry_add(obj);
			elm_entry_single_line_set(ad->gen_entry, EINA_TRUE);
			elm_entry_scrollable_set(ad->gen_entry, EINA_TRUE);
			if (!strcmp(data, "Latitude : ")) {
				snprintf(buf, sizeof(buf), "%f", LATITUDE);
				elm_entry_entry_set(ad->gen_entry, buf);
			} else if (!strcmp(data, "Bssid : "))
				elm_entry_entry_set(ad->gen_entry, BSSID);
			else
				elm_entry_entry_set(ad->gen_entry, value);

			return ad->gen_entry;
		}
	}
	return NULL;
}

static void
showError(Evas_Object *obj, int ret)
{
	switch (ret) {
	case GEOFENCE_MANAGER_ERROR_NONE:
		elm_entry_entry_set(obj, "<font_size=25><align=left>GEOFENCE_MANAGER_ERROR_NONE</align></font_size>");
		break;
	case GEOFENCE_MANAGER_ERROR_OUT_OF_MEMORY:
		elm_entry_entry_set(obj, "<font_size=25><align=left>GEOFENCE_MANAGER_ERROR_OUT_OF_MEMORY</align></font_size>");
		break;
	case GEOFENCE_MANAGER_ERROR_INVALID_PARAMETER:
		elm_entry_entry_set(obj, "<font_size=25><align=left>GEOFENCE_MANAGER_ERROR_INVALID_PARAMETER</align></font_size>");
		break;
	case GEOFENCE_MANAGER_ERROR_PERMISSION_DENIED:
		elm_entry_entry_set(obj, "<font_size=25><align=left>GEOFENCE_MANAGER_ERROR_PERMISSION_DENIED</align></font_size>");
		break;
	case GEOFENCE_MANAGER_ERROR_NOT_SUPPORTED:
		elm_entry_entry_set(obj, "<font_size=25><align=left>GEOFENCE_MANAGER_ERROR_NOT_SUPPORTED</align></font_size>");
		break;
	case GEOFENCE_MANAGER_ERROR_NOT_INITIALIZED:
		elm_entry_entry_set(obj, "<font_size=25><align=left>GEOFENCE_MANAGER_ERROR_NOT_INITIALIZED</align></font_size>");
		break;
	case GEOFENCE_MANAGER_ERROR_INVALID_ID:
		elm_entry_entry_set(obj, "<font_size=25><align=left>GEOFENCE_MANAGER_ERROR_INVALID_ID</align></font_size>");
		break;
	case GEOFENCE_MANAGER_ERROR_EXCEPTION:
		elm_entry_entry_set(obj, "<font_size=25><align=left>GEOFENCE_MANAGER_ERROR_EXCEPTION</align></font_size>");
		break;
	case GEOFENCE_MANAGER_ERROR_ALREADY_STARTED:
		elm_entry_entry_set(obj, "<font_size=25><align=left>GEOFENCE_MANAGER_ERROR_ALREADY_STARTED</align></font_size>");
		break;
	case GEOFENCE_MANAGER_ERROR_TOO_MANY_GEOFENCE:
		elm_entry_entry_set(obj, "<font_size=25><align=left>GEOFENCE_MANAGER_ERROR_TOO_MANY_GEOFENCE</align></font_size>");
		break;
	case GEOFENCE_MANAGER_ERROR_IPC:
		elm_entry_entry_set(obj, "<font_size=25><align=left>GEOFENCE_MANAGER_ERROR_IPC</align></font_size>");
		break;
	case GEOFENCE_MANAGER_ERROR_DATABASE:
		elm_entry_entry_set(obj, "<font_size=25><align=left>GEOFENCE_MANAGER_ERROR_DATABASE</align></font_size>");
		break;
	case GEOFENCE_MANAGER_ERROR_PLACE_ACCESS_DENIED:
		elm_entry_entry_set(obj, "<font_size=25><align=left>GEOFENCE_MANAGER_ERROR_PLACE_ACCESS_DENIED</align></font_size>");
		break;
	case GEOFENCE_MANAGER_ERROR_GEOFENCE_ACCESS_DENIED:
		elm_entry_entry_set(obj, "<font_size=25><align=left>GEOFENCE_MANAGER_ERROR_GEOFENCE_ACCESS_DENIED</align></font_size>");
		break;
	default:
		elm_entry_entry_set(obj, "<font_size=25><align=left>FAILED with UNKNOWN ERROR</align></font_size>");
		break;
	}
}

void
show_event_cb(int place_id, int geofence_id, geofence_manager_error_e error, geofence_manage_e manage, void *user_data)
{
	appdata_s *ad = (appdata_s *)user_data;
	char buf[10] = {};

	if (error == GEOFENCE_MANAGER_ERROR_NONE) {
		switch (manage) {
		case GEOFENCE_MANAGE_FENCE_ADDED:
			elm_entry_entry_set(ad->event_entry, "<font_size=25><align=left>GEOFENCE_MANAGE_FENCE_ADDED</align></font_size>");
			snprintf(buf, sizeof(buf), " : %d", geofence_id);
			break;
		case GEOFENCE_MANAGE_FENCE_REMOVED:
			elm_entry_entry_set(ad->event_entry, "<font_size=25><align=left>GEOFENCE_MANAGE_FENCE_REMOVED</align></font_size>");
			snprintf(buf, sizeof(buf), " : %d", geofence_id);
			break;
		case GEOFENCE_MANAGE_FENCE_STARTED:
			elm_entry_entry_set(ad->event_entry, "<font_size=25><align=left>GEOFENCE_MANAGE_FENCE_STARTED</align></font_size>");
			snprintf(buf, sizeof(buf), " : %d", geofence_id);
			break;
		case GEOFENCE_MANAGE_FENCE_STOPPED:
			elm_entry_entry_set(ad->event_entry, "<font_size=25><align=left>GEOFENCE_MANAGE_FENCE_STOPPED</align></font_size>");
			snprintf(buf, sizeof(buf), " : %d", geofence_id);
			break;
		case GEOFENCE_MANAGE_PLACE_ADDED:
			elm_entry_entry_set(ad->event_entry, "<font_size=25><align=left>GEOFENCE_MANAGE_PLACE_ADDED</align></font_size>");
			snprintf(buf, sizeof(buf), " : %d", place_id);
			break;
		case GEOFENCE_MANAGE_PLACE_REMOVED:
			elm_entry_entry_set(ad->event_entry, "<font_size=25><align=left>GEOFENCE_MANAGE_PLACE_REMOVED</align></font_size>");
			snprintf(buf, sizeof(buf), " : %d", place_id);
			break;
		case GEOFENCE_MANAGE_PLACE_UPDATED:
			elm_entry_entry_set(ad->event_entry, "<font_size=25><align=left>GEOFENCE_MANAGE_PLACE_UPDATED</align></font_size>");
			snprintf(buf, sizeof(buf), " : %d", place_id);
			break;
		case GEOFENCE_MANAGE_SETTING_ENABLED:
			elm_entry_entry_set(ad->event_entry, "<font_size=25><align=left>GEOFENCE_MANAGE_SETTING_ENABLED</align></font_size>");
			break;
		case GEOFENCE_MANAGE_SETTING_DISABLED:
			elm_entry_entry_set(ad->event_entry, "<font_size=25><align=left>GEOFENCE_MANAGE_SETTING_DISABLED</align></font_size>");
			break;
		default:
			elm_entry_entry_set(ad->event_entry, "<font_size=25><align=left>GEOFENCE_MANAGER_ERROR_NONE</align></font_size>");
			break;
		}
		elm_entry_entry_append(ad->event_entry, "<font_size=30>");
		elm_entry_entry_append(ad->event_entry, buf);
		elm_entry_entry_append(ad->event_entry, "</font_size>");
	} else
		showError(ad->event_entry, error);
}

void
show_state_changed_cb(int geofence_id, geofence_state_e state, void *user_data)
{
	appdata_s *ad = (appdata_s *)user_data;
	char buf[10] = {};

	if (state == GEOFENCE_STATE_IN)
		elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left>GEOFENCE_STATE_IN");
	else if (state == GEOFENCE_STATE_OUT)
		elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left>GEOFENCE_STATE_OUT");
	else
		elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left>GEOFENCE_STATE_UNCERTAIN");

	snprintf(buf, sizeof(buf), " : %d", geofence_id);
	elm_entry_entry_append(ad->status_entry, buf);
	elm_entry_entry_append(ad->status_entry, "</align></font_size>");
}

bool
get_place_cb(int place_id, const char *place_name, int place_index, int place_cnt, void *user_data)
{
	appdata_s *ad = (appdata_s *)user_data;

	ad->place_id[place_index-1] = place_id;

	return true;
}

static void
start_fence_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = evas_object_data_get(obj, "app_data");
	int fence_id = (int) data;
	int ret = 0;

	evas_object_del(ad->popup);

	ret = geofence_manager_start(ad->geo_manager, fence_id);

	elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_manager_start</align></font_size>");
	showError(ad->event_entry, ret);
	elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left></align></font_size>");
}

static void
stop_fence_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = evas_object_data_get(obj, "app_data");
	int fence_id = (int) data;
	int ret = 0;

	evas_object_del(ad->popup);

	ret = geofence_manager_stop(ad->geo_manager, fence_id);

	elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_manager_stop</align></font_size>");
	showError(ad->event_entry, ret);
	elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left></align></font_size>");
}

static void
remove_fence_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = evas_object_data_get(obj, "app_data");
	int fence_id = (int) data;
	int ret = 0;

	evas_object_del(ad->popup);

	ret = geofence_manager_remove_fence(ad->geo_manager, fence_id);

	elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_manager_remove_fence</align></font_size>");
	showError(ad->event_entry, ret);
	elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left></align></font_size>");
}

static void
get_status_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = evas_object_data_get(obj, "app_data");
	int geofence_id = (int) data;
	geofence_status_h status = NULL;
	geofence_state_e state;
	int seconds = 0;
	int ret = 0;
	char buf[50] = {};
	char *state_str[] = {"UNCERTAIN", "IN", "OUT"};

	evas_object_del(ad->popup);

	ret = geofence_status_create(geofence_id, &status);
	elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_status_create</align></font_size>");
	showError(ad->event_entry, ret);
	elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left></align></font_size>");

	if (ret != GEOFENCE_MANAGER_ERROR_NONE)
		return;

	ret = geofence_status_get_state(status, &state);
	elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_status_get_state</align></font_size>");
	showError(ad->event_entry, ret);
	elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left></align></font_size>");

	ret = geofence_status_get_duration(status, &seconds);
	elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_status_get_duration</align></font_size>");
	showError(ad->event_entry, ret);
	elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left></align></font_size>");

	if (ret != GEOFENCE_MANAGER_ERROR_NONE)
		return;

	ret = geofence_status_destroy(status);
	elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_status_destroy</align></font_size>");
	showError(ad->event_entry, ret);
	snprintf(buf, sizeof(buf), "Fence ID: %d, State: %s, Duration: %d", geofence_id, state_str[state], seconds);
	elm_entry_entry_append(ad->status_entry, "<font_size=25><align=left>");
	elm_entry_entry_append(ad->status_entry, buf);
	elm_entry_entry_append(ad->status_entry, "</align></font_size>");
}

static void
popup_cancel_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = (appdata_s *)data;
	evas_object_del(ad->popup);
}

static void
popup_place_name_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = evas_object_data_get(obj, "app_data");
	int ret = 0;
	int place_id = (int)data;
	const char *place_name = elm_entry_entry_get(ad->gen_entry);

	if (!strcmp(place_name, "")) {
		show_toast_popup(ad, "Empty Value");
		return;
	}

	if (place_id < 0) {
		ret = geofence_manager_add_place(ad->geo_manager, place_name, &place_id);
		elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_manager_add_place</align></font_size>");
	} else {
		ret = geofence_manager_update_place(ad->geo_manager, place_id, place_name);
		elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_manager_update_place</align></font_size>");
	}

	showError(ad->event_entry, ret);

	if (place_id > 0 && place_id <= 3)
		elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left>The renaming of default place is impossible</align></font_size>");
	else
		elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left></align></font_size>");

	evas_object_del(ad->popup);
}

static void
popup_method_value_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = evas_object_data_get(obj, "app_data");
	int place_id = (int)evas_object_data_get(obj, "place_id");;
	int method = (int)data;

	const char *value = elm_entry_entry_get(ad->gen_entry);
	const char *value_nd = elm_entry_entry_get(ad->gen_entry_nd);

	geofence_h fence = NULL;
	double latitude = 0.0, longitude = 0.0;
	int geofence_id = -1;
	int ret = 0;

	if (!strcmp(value, "")) {
		show_toast_popup(ad, "Empty Value");
		return;
	}
	if (method == METHOD_GPS) {
		if (!strcmp(value_nd, "")) {
			show_toast_popup(ad, "Empty Value");
			return;
		}
	}

	switch (method) {
	case METHOD_GPS:
		latitude = atof(value);
		longitude = atof(value_nd);
		ret = geofence_create_geopoint(place_id, latitude, longitude, 200, "Dubai MR", &fence);
		elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_create_geopoint</align></font_size>");
		break;
	case METHOD_WIFI:
		ret = geofence_create_wifi(place_id, value, "Dubai MR", &fence);
		elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_create_wifi</align></font_size>");
		break;
	case METHOD_BT:
		ret = geofence_create_bluetooth(place_id, value, "Dubai MR", &fence);
		elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_create_bluetooth</align></font_size>");
		break;
	default:
		break;
	}
	showError(ad->event_entry, ret);
	elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left></align></font_size>");

	ret = geofence_manager_add_fence(ad->geo_manager, fence, &geofence_id);
	showError(ad->event_entry, ret);

	evas_object_del(ad->popup);
}

static void
insert_fence_value(appdata_s *ad, int place_id, int method)
{
	Elm_Genlist_Item_Class *itc;
	Evas_Object *box;
	Evas_Object *genlist;
	Evas_Object *btn;
	int ret = 0;

	ad->popup = elm_popup_add(ad->nf);
	elm_popup_align_set(ad->popup, ELM_NOTIFY_ALIGN_FILL, 1.0);
	eext_object_event_callback_add(ad->popup, EEXT_CALLBACK_BACK, eext_popup_back_cb, NULL);
	evas_object_size_hint_weight_set(ad->popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	if (method == METHOD_GPS)
		elm_object_part_text_set(ad->popup, "title,text", "GPS");
	else if (method == METHOD_WIFI)
		elm_object_part_text_set(ad->popup, "title,text", "Wi-Fi");
	else
		elm_object_part_text_set(ad->popup, "title,text", "Bluetooth");

	/* box */
	box = elm_box_add(ad->popup);
	evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	/* genlist */
	genlist = elm_genlist_add(box);
	evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);

	evas_object_data_set(genlist, "app_data", ad);

	itc = elm_genlist_item_class_new();
	itc->item_style = "entry.sub";
	itc->func.text_get = gl_text_get_cb;
	itc->func.content_get = gl_entry_content_get;
	itc->func.state_get = NULL;
	itc->func.del = NULL;

	if (method == METHOD_GPS) {
		elm_genlist_item_append(genlist, itc, "Latitude : ", NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
		elm_genlist_item_append(genlist, itc, "Longitude : ", NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
	} else
		elm_genlist_item_append(genlist, itc, "Bssid : ", NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);

	evas_object_show(genlist);
	elm_box_pack_end(box, genlist);
	if (method == METHOD_GPS)
		evas_object_size_hint_min_set(box, -1, 300);
	else
		evas_object_size_hint_min_set(box, -1, 150);
	elm_object_content_set(ad->popup, box);

	/* Cancel button */
	btn = elm_button_add(ad->popup);
	elm_object_style_set(btn, "popup");
	elm_object_text_set(btn, "Cancel");
	elm_object_part_content_set(ad->popup, "button1", btn);
	evas_object_smart_callback_add(btn, "clicked", popup_cancel_cb, ad);

	/* Done button */
	btn = elm_button_add(ad->popup);
	evas_object_data_set(btn, "app_data", ad);
	evas_object_data_set(btn, "place_id", (void *)place_id);
	elm_object_style_set(btn, "popup");
	elm_object_text_set(btn, "Done");
	elm_object_part_content_set(ad->popup, "button2", btn);
	evas_object_smart_callback_add(btn, "clicked", popup_method_value_cb, (void *)method);

	evas_object_show(ad->popup);
	elm_genlist_item_class_free(itc);
}

bool
get_fence_cb(int geofence_id, geofence_h fence, int fence_index, int fence_cnt, void *user_data)
{
	appdata_s *ad = (appdata_s *)user_data;

	ad->fence_id[fence_index-1] = geofence_id;

	return true;
}

bool
get_started_fence_cb(int geofence_id, geofence_h fence, int fence_index, int fence_cnt, void *user_data)
{
	appdata_s *ad = (appdata_s *)user_data;
	geofence_status_h status = NULL;
	geofence_state_e state;
	int ret = 0;

	ret = geofence_status_create(geofence_id, &status);
	elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_status_create</align></font_size>");
	showError(ad->event_entry, ret);
	elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left></align></font_size>");

	if (ret != GEOFENCE_MANAGER_ERROR_NONE)
		return false;

	ret = geofence_status_get_state(status, &state);
	elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_status_get_state</align></font_size>");
	showError(ad->event_entry, ret);
	elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left></align></font_size>");

	if (ret != GEOFENCE_MANAGER_ERROR_NONE)
		return false;

	if (state != GEOFENCE_STATE_UNCERTAIN) {
		ad->fence_id[myfence_index] = geofence_id;
		myfence_index++;
	}

	ret = geofence_status_destroy(status);
	elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_status_destroy</align></font_size>");
	showError(ad->event_entry, ret);
	elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left></align></font_size>");

	if (ret != GEOFENCE_MANAGER_ERROR_NONE)
		return false;

	return true;
}

bool
get_stopped_fence_cb(int geofence_id, geofence_h fence, int fence_index, int fence_cnt, void *user_data)
{
	appdata_s *ad = (appdata_s *)user_data;
	geofence_status_h status = NULL;
	geofence_state_e state;
	int ret = 0;

	ret = geofence_status_create(geofence_id, &status);
	elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_status_create</align></font_size>");
	showError(ad->event_entry, ret);
	elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left></align></font_size>");

	if (ret != GEOFENCE_MANAGER_ERROR_NONE)
		return false;

	ret = geofence_status_get_state(status, &state);
	elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_status_get_state</align></font_size>");
	showError(ad->event_entry, ret);
	elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left></align></font_size>");

	if (ret != GEOFENCE_MANAGER_ERROR_NONE)
		return false;

	if (state == GEOFENCE_STATE_UNCERTAIN) {
		ad->fence_id[myfence_index] = geofence_id;
		myfence_index++;
	}

	ret = geofence_status_destroy(status);
	elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_status_destroy</align></font_size>");
	showError(ad->event_entry, ret);
	elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left></align></font_size>");

	if (ret != GEOFENCE_MANAGER_ERROR_NONE)
		return false;

	return true;
}

static void
select_method_list(appdata_s *ad, Evas_Smart_Cb func, int place_id)
{
	Elm_Genlist_Item_Class *itc_gps, *itc_wifi, *itc_bt;
	Elm_Object_Item *gi_gps, *gi_wifi, *gi_bt;
	Evas_Object *box;
	Evas_Object *genlist;

	const char *wifi_feature = "http://tizen.org/feature/network.wifi";
	const char *bluetooth_feature = "http://tizen.org/feature/network.bluetooth";

	bool wifi_supported = false;
	bool bt_supported = false;

	ad->popup = elm_popup_add(ad->nf);
	elm_popup_align_set(ad->popup, ELM_NOTIFY_ALIGN_FILL, 1.0);
	eext_object_event_callback_add(ad->popup, EEXT_CALLBACK_BACK, eext_popup_back_cb, NULL);
	evas_object_size_hint_weight_set(ad->popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_text_set(ad->popup, "title,text", "Method");

	/* box */
	box = elm_box_add(ad->popup);
	evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	/* genlist */
	genlist = elm_genlist_add(box);
	evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);

	evas_object_data_set(genlist, "app_data", ad);
	evas_object_data_set(genlist, "place_id", (void *)place_id);

	itc_gps = elm_genlist_item_class_new();
	itc_gps->item_style = "default";
	itc_gps->func.text_get = gl_method_text_get_cb;
	itc_gps->func.content_get = NULL;
	itc_gps->func.state_get = NULL;
	itc_gps->func.del = NULL;
	gi_gps = elm_genlist_item_append(genlist, itc_gps, (void *)METHOD_GPS, NULL, ELM_GENLIST_ITEM_NONE, func, (void *)METHOD_GPS);

	itc_wifi = elm_genlist_item_class_new();
	itc_wifi->item_style = "default";
	itc_wifi->func.text_get = gl_method_text_get_cb;
	itc_wifi->func.content_get = NULL;
	itc_wifi->func.state_get = NULL;
	itc_wifi->func.del = NULL;
	gi_wifi = elm_genlist_item_append(genlist, itc_wifi, (void *)METHOD_WIFI, NULL, ELM_GENLIST_ITEM_NONE, func, (void *)METHOD_WIFI);

	itc_bt = elm_genlist_item_class_new();
	itc_bt->item_style = "default";
	itc_bt->func.text_get = gl_method_text_get_cb;
	itc_bt->func.content_get = NULL;
	itc_bt->func.state_get = NULL;
	itc_bt->func.del = NULL;
	gi_bt = elm_genlist_item_append(genlist, itc_bt, (void *)METHOD_BT, NULL, ELM_GENLIST_ITEM_NONE, func, (void *)METHOD_BT);

	system_info_get_platform_bool(wifi_feature, &wifi_supported);
	if (wifi_supported == false)
		elm_object_item_disabled_set(gi_wifi, EINA_TRUE);
	else
		elm_object_item_disabled_set(gi_wifi, EINA_FALSE);
	system_info_get_platform_bool(bluetooth_feature, &bt_supported);
	if (bt_supported == false)
		elm_object_item_disabled_set(gi_bt, EINA_TRUE);
	else
		elm_object_item_disabled_set(gi_bt, EINA_FALSE);

	evas_object_show(genlist);
	elm_box_pack_end(box, genlist);
	evas_object_size_hint_min_set(box, -1, 300);
	elm_object_content_set(ad->popup, box);

	evas_object_show(ad->popup);
	elm_genlist_item_class_free(itc_gps);
	elm_genlist_item_class_free(itc_wifi);
	elm_genlist_item_class_free(itc_bt);
}

static void
select_fence_list(appdata_s *ad, Evas_Smart_Cb func)
{
	Elm_Genlist_Item_Class *itc;
	Evas_Object *box;
	Evas_Object *genlist;
	int ret = 0;
	int i = 0;
	myfence_index = 0;

	for (i = 0; i < MAX_COUNT; i++)
		ad->fence_id[i] = -1;

	if (func == start_fence_cb)
		ret = geofence_manager_foreach_geofence_list(ad->geo_manager, get_stopped_fence_cb, ad);
	else if (func == stop_fence_cb)
		ret = geofence_manager_foreach_geofence_list(ad->geo_manager, get_started_fence_cb, ad);
	else
		ret = geofence_manager_foreach_geofence_list(ad->geo_manager, get_fence_cb, ad);

	elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_manager_foreach_geofence_list</align></font_size>");
	showError(ad->event_entry, ret);
	elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left></align></font_size>");

	if (ret != GEOFENCE_MANAGER_ERROR_NONE)
		return;
	else if (ad->fence_id[0] == -1) {
		show_toast_popup(ad, "Empty");
		return;
	}

	ad->popup = elm_popup_add(ad->nf);
	elm_popup_align_set(ad->popup, ELM_NOTIFY_ALIGN_FILL, 1.0);
	eext_object_event_callback_add(ad->popup, EEXT_CALLBACK_BACK, eext_popup_back_cb, NULL);
	evas_object_size_hint_weight_set(ad->popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_text_set(ad->popup, "title,text", "Fence");

	/* box */
	box = elm_box_add(ad->popup);
	evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	/* genlist */
	genlist = elm_genlist_add(box);
	evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);

	evas_object_data_set(genlist, "app_data", ad);

	itc = elm_genlist_item_class_new();
	itc->item_style = "default";
	itc->func.text_get = gl_id_text_get_cb;
	itc->func.content_get = NULL;
	itc->func.state_get = NULL;
	itc->func.del = NULL;

	for (i = 0; ad->fence_id[i] != -1; i++)
		elm_genlist_item_append(genlist, itc, (void *)ad->fence_id[i], NULL, ELM_GENLIST_ITEM_NONE, func, (void *)ad->fence_id[i]);

	evas_object_show(genlist);
	elm_box_pack_end(box, genlist);
	evas_object_size_hint_min_set(box, -1, 150);
	elm_object_content_set(ad->popup, box);

	evas_object_show(ad->popup);
	elm_genlist_item_class_free(itc);
}

static void
insert_place_name(appdata_s *ad, Evas_Object *obj, int place_id, char *place_name)
{
	Elm_Genlist_Item_Class *itc;
	Evas_Object *box;
	Evas_Object *genlist;
	Evas_Object *btn;

	ad->popup = elm_popup_add(ad->nf);
	elm_popup_align_set(ad->popup, ELM_NOTIFY_ALIGN_FILL, 1.0);
	eext_object_event_callback_add(ad->popup, EEXT_CALLBACK_BACK, eext_popup_back_cb, NULL);
	evas_object_size_hint_weight_set(ad->popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	if (obj == ad->add_place_btn)
		elm_object_part_text_set(ad->popup, "title,text", "Add Place");
	else
		elm_object_part_text_set(ad->popup, "title,text", "Update Place");

	/* box */
	box = elm_box_add(ad->popup);
	evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	/* genlist */
	genlist = elm_genlist_add(box);
	evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);

	evas_object_data_set(genlist, "app_data", ad);
	evas_object_data_set(genlist, "value", place_name);

	itc = elm_genlist_item_class_new();
	itc->item_style = "entry.sub";
	itc->func.text_get = gl_text_get_cb;
	itc->func.content_get = gl_entry_content_get;
	itc->func.state_get = NULL;
	itc->func.del = NULL;

	elm_genlist_item_append(genlist, itc, "Name : ", NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);

	evas_object_show(genlist);
	elm_box_pack_end(box, genlist);
	evas_object_size_hint_min_set(box, -1, 150);
	elm_object_content_set(ad->popup, box);

	/* Cancel button */
	btn = elm_button_add(ad->popup);
	elm_object_style_set(btn, "popup");
	elm_object_text_set(btn, "Cancel");
	elm_object_part_content_set(ad->popup, "button1", btn);
	evas_object_smart_callback_add(btn, "clicked", popup_cancel_cb, ad);

	/* Done button */
	btn = elm_button_add(ad->popup);
	evas_object_data_set(btn, "app_data", ad);
	elm_object_style_set(btn, "popup");
	elm_object_text_set(btn, "Done");
	elm_object_part_content_set(ad->popup, "button2", btn);
	evas_object_smart_callback_add(btn, "clicked", popup_place_name_cb, (void *)place_id);

	evas_object_show(ad->popup);
	elm_genlist_item_class_free(itc);
}

static void
add_fence_nd_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = evas_object_data_get(obj, "app_data");
	int place_id = (int)evas_object_data_get(obj, "place_id");
	int method = (int) data;

	evas_object_del(ad->popup);
	insert_fence_value(ad, place_id, method);
}

static void
add_fence_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = evas_object_data_get(obj, "app_data");
	int place_id = (int) data;

	evas_object_del(ad->popup);
	select_method_list(ad, add_fence_nd_cb, place_id);
}

static void
remove_place_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = evas_object_data_get(obj, "app_data");
	int place_id = (int) data;
	int ret = 0;

	evas_object_del(ad->popup);

	ret = geofence_manager_remove_place(ad->geo_manager, place_id);

	elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_manager_remove_place</align></font_size>");
	showError(ad->event_entry, ret);
	if (place_id <= 3)
		elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left>The deletion of default place is impossible</align></font_size>");
	else
		elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left></align></font_size>");
}

static void
update_place_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = evas_object_data_get(obj, "app_data");
	int place_id = (int) data;
	char *name = NULL;
	int ret = 0;

	evas_object_del(ad->popup);

	ret = geofence_manager_get_place_name(ad->geo_manager, place_id, &name);

	elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_manager_get_place_name</align></font_size>");
	showError(ad->event_entry, ret);
	elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left></align></font_size>");

	insert_place_name(ad, ad->update_place_btn, place_id, name);
}

static void
select_place_list(appdata_s *ad, Evas_Smart_Cb func)
{
	Elm_Genlist_Item_Class *itc;
	Evas_Object *box;
	Evas_Object *genlist;
	int ret = 0;
	int i = 0;

	for (i = 0; i < MAX_COUNT; i++)
		ad->place_id[i] = -1;

	ret = geofence_manager_foreach_place_list(ad->geo_manager, get_place_cb, ad);

	elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_manager_foreach_place_list</align></font_size>");
	showError(ad->event_entry, ret);
	elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left></align></font_size>");

	if (ret != GEOFENCE_MANAGER_ERROR_NONE)
		return;

	ad->popup = elm_popup_add(ad->nf);
	elm_popup_align_set(ad->popup, ELM_NOTIFY_ALIGN_FILL, 1.0);
	eext_object_event_callback_add(ad->popup, EEXT_CALLBACK_BACK, eext_popup_back_cb, NULL);
	evas_object_size_hint_weight_set(ad->popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_text_set(ad->popup, "title,text", "Place");

	/* box */
	box = elm_box_add(ad->popup);
	evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	/* genlist */
	genlist = elm_genlist_add(box);
	evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);

	evas_object_data_set(genlist, "app_data", ad);

	itc = elm_genlist_item_class_new();
	itc->item_style = "default";
	itc->func.text_get = gl_id_text_get_cb;
	itc->func.content_get = NULL;
	itc->func.state_get = NULL;
	itc->func.del = NULL;

	for (i = 0; ad->place_id[i] != -1; i++)
		elm_genlist_item_append(genlist, itc, (void *)ad->place_id[i], NULL, ELM_GENLIST_ITEM_NONE, func, (void *)ad->place_id[i]);

	evas_object_show(genlist);
	elm_box_pack_end(box, genlist);
	evas_object_size_hint_min_set(box, -1, 300);
	elm_object_content_set(ad->popup, box);

	evas_object_show(ad->popup);
	elm_genlist_item_class_free(itc);
}

static void
clicked_create_cb(void *data, Evas_Object *obj EINA_UNUSED, void *event_info EINA_UNUSED)
{
	appdata_s *ad = (appdata_s *) data;
	int ret = 0;

	ret = geofence_manager_create(&(ad->geo_manager));

	elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_manager_create</align></font_size>");
	showError(ad->event_entry, ret);
	elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left></align></font_size>");

	if (ret == GEOFENCE_MANAGER_ERROR_NONE) {
		elm_object_disabled_set(ad->create_btn, EINA_TRUE);
		elm_object_disabled_set(ad->fence_status_btn, EINA_FALSE);
		elm_object_disabled_set(ad->add_fence_btn, EINA_FALSE);
		elm_object_disabled_set(ad->add_place_btn, EINA_FALSE);
		elm_object_disabled_set(ad->start_btn, EINA_FALSE);
		elm_object_disabled_set(ad->stop_btn, EINA_FALSE);
		elm_object_disabled_set(ad->remove_fence_btn, EINA_FALSE);
		elm_object_disabled_set(ad->remove_place_btn, EINA_FALSE);
		elm_object_disabled_set(ad->update_place_btn, EINA_FALSE);
		elm_object_disabled_set(ad->destroy_btn, EINA_FALSE);

		geofence_manager_set_geofence_event_cb(ad->geo_manager, show_event_cb, ad);
		geofence_manager_set_geofence_state_changed_cb(ad->geo_manager, show_state_changed_cb, ad);
	}
}

static void
clicked_destroy_cb(void *data, Evas_Object *obj EINA_UNUSED, void *event_info EINA_UNUSED)
{
	appdata_s *ad = (appdata_s *) data;
	int ret = 0;

	geofence_manager_unset_geofence_event_cb(ad->geo_manager);
	geofence_manager_unset_geofence_state_changed_cb(ad->geo_manager);

	ret = geofence_manager_destroy(ad->geo_manager);

	elm_entry_entry_set(ad->entry, "<font_size=30><align=left>geofence_manager_destroy</align></font_size>");
	showError(ad->event_entry, ret);
	elm_entry_entry_set(ad->status_entry, "<font_size=25><align=left></align></font_size>");

	if (ret == GEOFENCE_MANAGER_ERROR_NONE) {
		elm_object_disabled_set(ad->create_btn, EINA_FALSE);
		elm_object_disabled_set(ad->fence_status_btn, EINA_TRUE);
		elm_object_disabled_set(ad->add_fence_btn, EINA_TRUE);
		elm_object_disabled_set(ad->add_place_btn, EINA_TRUE);
		elm_object_disabled_set(ad->start_btn, EINA_TRUE);
		elm_object_disabled_set(ad->stop_btn, EINA_TRUE);
		elm_object_disabled_set(ad->remove_fence_btn, EINA_TRUE);
		elm_object_disabled_set(ad->remove_place_btn, EINA_TRUE);
		elm_object_disabled_set(ad->update_place_btn, EINA_TRUE);
		elm_object_disabled_set(ad->destroy_btn, EINA_TRUE);

		ad->geo_manager = NULL;
	}
}

static void
show_popup_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = (appdata_s *)data;

	if (obj == ad->fence_status_btn)
		select_fence_list(ad, get_status_cb);
	else if (obj == ad->add_fence_btn)
		select_place_list(ad, add_fence_cb);
	else if (obj == ad->add_place_btn)
		insert_place_name(ad, ad->add_place_btn, -1, NULL);
	else if (obj == ad->start_btn)
		select_fence_list(ad, start_fence_cb);
	else if (obj == ad->stop_btn)
		select_fence_list(ad, stop_fence_cb);
	else if (obj == ad->remove_fence_btn)
		select_fence_list(ad, remove_fence_cb);
	else if (obj == ad->remove_place_btn)
		select_place_list(ad, remove_place_cb);
	else if (obj == ad->update_place_btn)
		select_place_list(ad, update_place_cb);
}

/****************************** Main UI functions ******************************/
static Evas_Object *
create_panel_basic_content(Evas_Object *parent, appdata_s *ad)
{
	Evas_Object *table;

	table = elm_table_add(parent);
	elm_table_padding_set(table, 10, 0);

	ad->create_btn = elm_button_add(table);
	evas_object_size_hint_weight_set(ad->create_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(ad->create_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_text_set(ad->create_btn, "<font_size=30>Create</font_size>");
	elm_object_disabled_set(ad->create_btn, EINA_FALSE);
	evas_object_smart_callback_add(ad->create_btn, "clicked", clicked_create_cb, ad);
	evas_object_show(ad->create_btn);
	elm_table_pack(table, ad->create_btn, 0, 0, 1, 1);

	ad->fence_status_btn = elm_button_add(table);
	evas_object_size_hint_weight_set(ad->fence_status_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(ad->fence_status_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_text_set(ad->fence_status_btn, "<font_size=30>Fence Status</font_size>");
	elm_object_disabled_set(ad->fence_status_btn, EINA_TRUE);
	evas_object_smart_callback_add(ad->fence_status_btn, "clicked", show_popup_cb, ad);
	evas_object_show(ad->fence_status_btn);
	elm_table_pack(table, ad->fence_status_btn, 1, 0, 1, 1);

	ad->add_fence_btn = elm_button_add(table);
	evas_object_size_hint_weight_set(ad->add_fence_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(ad->add_fence_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_text_set(ad->add_fence_btn, "<font_size=30>Add Fence</font_size>");
	elm_object_disabled_set(ad->add_fence_btn, EINA_TRUE);
	evas_object_smart_callback_add(ad->add_fence_btn, "clicked", show_popup_cb, ad);
	evas_object_show(ad->add_fence_btn);
	elm_table_pack(table, ad->add_fence_btn, 0, 1, 1, 1);

	ad->add_place_btn = elm_button_add(table);
	evas_object_size_hint_weight_set(ad->add_place_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(ad->add_place_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_text_set(ad->add_place_btn, "<font_size=30>Add Place</font_size>");
	elm_object_disabled_set(ad->add_place_btn, EINA_TRUE);
	evas_object_smart_callback_add(ad->add_place_btn, "clicked", show_popup_cb, ad);
	evas_object_show(ad->add_place_btn);
	elm_table_pack(table, ad->add_place_btn, 1, 1, 1, 1);

	ad->start_btn = elm_button_add(table);
	evas_object_size_hint_weight_set(ad->start_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(ad->start_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_text_set(ad->start_btn, "<font_size=30>Start</font_size>");
	elm_object_disabled_set(ad->start_btn, EINA_TRUE);
	evas_object_smart_callback_add(ad->start_btn, "clicked", show_popup_cb, ad);
	evas_object_show(ad->start_btn);
	elm_table_pack(table, ad->start_btn, 0, 2, 1, 1);

	ad->stop_btn = elm_button_add(table);
	evas_object_size_hint_weight_set(ad->stop_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(ad->stop_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_text_set(ad->stop_btn, "<font_size=30>Stop</font_size>");
	elm_object_disabled_set(ad->stop_btn, EINA_TRUE);
	evas_object_smart_callback_add(ad->stop_btn, "clicked", show_popup_cb, ad);
	evas_object_show(ad->stop_btn);
	elm_table_pack(table, ad->stop_btn, 1, 2, 1, 1);

	ad->remove_fence_btn = elm_button_add(table);
	evas_object_size_hint_weight_set(ad->remove_fence_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(ad->remove_fence_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_text_set(ad->remove_fence_btn, "<font_size=30>Remove Fence</font_size>");
	elm_object_disabled_set(ad->remove_fence_btn, EINA_TRUE);
	evas_object_smart_callback_add(ad->remove_fence_btn, "clicked", show_popup_cb, ad);
	evas_object_show(ad->remove_fence_btn);
	elm_table_pack(table, ad->remove_fence_btn, 0, 3, 1, 1);

	ad->remove_place_btn = elm_button_add(table);
	evas_object_size_hint_weight_set(ad->remove_place_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(ad->remove_place_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_text_set(ad->remove_place_btn, "<font_size=30>Remove Place</font_size>");
	elm_object_disabled_set(ad->remove_place_btn, EINA_TRUE);
	evas_object_smart_callback_add(ad->remove_place_btn, "clicked", show_popup_cb, ad);
	evas_object_show(ad->remove_place_btn);
	elm_table_pack(table, ad->remove_place_btn, 1, 3, 1, 1);

	ad->update_place_btn = elm_button_add(table);
	evas_object_size_hint_weight_set(ad->update_place_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(ad->update_place_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_text_set(ad->update_place_btn, "<font_size=30>Update Place</font_size>");
	elm_object_disabled_set(ad->update_place_btn, EINA_TRUE);
	evas_object_smart_callback_add(ad->update_place_btn, "clicked", show_popup_cb, ad);
	evas_object_show(ad->update_place_btn);
	elm_table_pack(table, ad->update_place_btn, 0, 4, 1, 1);

	ad->destroy_btn = elm_button_add(table);
	evas_object_size_hint_weight_set(ad->destroy_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(ad->destroy_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_text_set(ad->destroy_btn, "<font_size=30>Destroy</font_size>");
	elm_object_disabled_set(ad->destroy_btn, EINA_TRUE);
	evas_object_smart_callback_add(ad->destroy_btn, "clicked", clicked_destroy_cb, ad);
	evas_object_show(ad->destroy_btn);
	elm_table_pack(table, ad->destroy_btn, 1, 4, 1, 1);

	evas_object_show(table);

	return table;
}

static Evas_Object *
create_panel(Evas_Object *parent, appdata_s *ad)
{
	Evas_Object *panel, *grid;

	/* Panel */
	panel = elm_panel_add(parent);
	evas_object_show(panel);

	/* Grid */
	grid = elm_grid_add(panel);
	evas_object_size_hint_weight_set(grid, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(grid, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_content_set(panel, grid);

	/* Panel basic content */
	ad->basic_content = create_panel_basic_content(grid, ad);
	evas_object_size_hint_weight_set(ad->basic_content, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(ad->basic_content, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_grid_pack(grid, ad->basic_content, 3, 3, 94, 94);

	return panel;
}

static Evas_Object *
create_entry(Evas_Object *parent)
{
	Evas_Object *entry;

	entry = elm_entry_add(parent);
	elm_entry_editable_set(entry, EINA_FALSE);
	elm_entry_context_menu_disabled_set(entry, EINA_TRUE);
	elm_entry_entry_set(entry, "<font_size=30><align=left></align></font_size>");
	evas_object_show(entry);

	return entry;
}

static void
create_base_gui(appdata_s *ad)
{
	Evas_Object *grid, *panel;
	Elm_Object_Item *nf_it;

	/* Window */
	ad->win = elm_win_util_standard_add(PACKAGE, PACKAGE);
	elm_win_conformant_set(ad->win, EINA_TRUE);
	elm_win_autodel_set(ad->win, EINA_TRUE);
	evas_object_smart_callback_add(ad->win, "delete,request", win_delete_request_cb, NULL);

	/* Conformant */
	ad->conform = elm_conformant_add(ad->win);
	evas_object_size_hint_weight_set(ad->conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(ad->win, ad->conform);
	evas_object_show(ad->conform);

	/* Indicator */
	elm_win_indicator_mode_set(ad->win, ELM_WIN_INDICATOR_SHOW);
	elm_win_indicator_opacity_set(ad->win, ELM_WIN_INDICATOR_OPAQUE);

	/* layout */
	ad->layout = elm_layout_add(ad->conform);
	evas_object_size_hint_weight_set(ad->layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_layout_theme_set(ad->layout, "layout", "application", "default");
	elm_object_content_set(ad->conform, ad->layout);
	evas_object_show(ad->layout);

	ad->nf = elm_naviframe_add(ad->layout);
	elm_object_part_content_set(ad->layout, "elm.swallow.content", ad->nf);
	eext_object_event_callback_add(ad->nf, EEXT_CALLBACK_BACK, eext_naviframe_back_cb, NULL);
	evas_object_show(ad->nf);

	/* Grid */
	grid = elm_grid_add(ad->nf);
	evas_object_size_hint_weight_set(grid, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(grid, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_content_set(ad->nf, grid);

	/* Entry */
	ad->entry = create_entry(grid);
	elm_grid_pack(grid, ad->entry, 0, 5, 90, 10);

	ad->event_entry = create_entry(grid);
	elm_grid_pack(grid, ad->event_entry, 0, 15, 90, 10);

	ad->status_entry = create_entry(grid);
	elm_grid_pack(grid, ad->status_entry, 0, 25, 90, 10);

	/* Panel */
	panel = create_panel(grid, ad);
	elm_grid_pack(grid, panel, 0, 55, 100, 45);

	nf_it = elm_naviframe_item_push(ad->nf, "Geofence", NULL, NULL, grid, NULL);
	elm_naviframe_item_pop_cb_set(nf_it, win_delete_request_cb, ad);
	/* Show window after base gui is set up */
	evas_object_show(ad->win);
}

/****************************** default functions ******************************/
static void
app_control(app_control_h app_control, void *data)
{
	/* Handle the launch request. */
}

static void
app_pause(void *data)
{
	/* Take necessary actions when application becomes invisible. */
}

static void
app_resume(void *data)
{
	/* Take necessary actions when application becomes visible. */
}

static void
app_terminate(void *user_data)
{
    /* Release all resources (*/
	appdata_s *ad = user_data;

	if (!user_data)
		return;

	if (ad->geo_manager != NULL)
		geofence_manager_destroy(ad->geo_manager);
	ad->geo_manager = NULL;

	ad = user_data;

	if (ad->win)
		evas_object_del(ad->win);
}

static void
ui_app_lang_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LANGUAGE_CHANGED*/
	char *locale = NULL;
	system_settings_get_value_string(SYSTEM_SETTINGS_KEY_LOCALE_LANGUAGE, &locale);
	elm_language_set(locale);
	free(locale);
	return;
}

static void
ui_app_orient_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_DEVICE_ORIENTATION_CHANGED*/
	return;
}

static void
ui_app_region_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_REGION_FORMAT_CHANGED*/
}

static void
ui_app_low_battery(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LOW_BATTERY*/
}

static void
ui_app_low_memory(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LOW_MEMORY*/
}

static bool
app_create(void *data)
{
	/* Hook to take necessary actions before main event loop starts
		Initialize UI resources and application's data
		If this function returns true, the main loop of application starts
		If this function returns false, the application is terminated */
	appdata_s *ad = data;

	create_base_gui(ad);

	return true;
}

int main(int argc, char *argv[])
{
	appdata_s ad = {0,};
	int ret = 0;

	ui_app_lifecycle_callback_s event_callback = {0,};
	app_event_handler_h handlers[5] = {NULL, };

	event_callback.create = app_create;
	event_callback.terminate = app_terminate;
	event_callback.pause = app_pause;
	event_callback.resume = app_resume;
	event_callback.app_control = app_control;

	ui_app_add_event_handler(&handlers[APP_EVENT_LOW_BATTERY], APP_EVENT_LOW_BATTERY, ui_app_low_battery, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_LOW_MEMORY], APP_EVENT_LOW_MEMORY, ui_app_low_memory, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_DEVICE_ORIENTATION_CHANGED], APP_EVENT_DEVICE_ORIENTATION_CHANGED, ui_app_orient_changed, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED], APP_EVENT_LANGUAGE_CHANGED, ui_app_lang_changed, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_REGION_FORMAT_CHANGED], APP_EVENT_REGION_FORMAT_CHANGED, ui_app_region_changed, &ad);
	ui_app_remove_event_handler(handlers[APP_EVENT_LOW_MEMORY]);

	ret = ui_app_main(argc, argv, &event_callback, &ad);
	if (ret != APP_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "ui_app_main() is failed. err = %d", ret);

	return ret;
}
